#include <stdbool.h>
#include <string.h>


#ifndef EVENT_CONFIG_H_
#define EVENT_CONFIG_H_

#define MIN_VERSION 1
#define MAX_VERSION 2

typedef enum {
    ALL = 0,
    AUDIO = 1,
    PHOTO = 2,
    VIDEO = 3
} media_type_t;

#define MEDIA_TYPE_VIDEO "video"
#define MEDIA_TYPE_AUDIO "audio"
#define MEDIA_TYPE_PHOTO "photo"
#define MEDIA_TYPE_ALL   "all"

#define mt_is_video(str) (strcmp(str, MEDIA_TYPE_VIDEO) == 0)
#define mt_is_audio(str) (strcmp(str, MEDIA_TYPE_AUDIO) == 0)
#define mt_is_photo(str) (strcmp(str, MEDIA_TYPE_PHOTO) == 0)
#define mt_is_all(str)   (strcmp(str, MEDIA_TYPE_ALL) == 0)


typedef enum {
    DIRECTORY_CHANGE,
} listener_type_t;

#define LISTENER_TYPE_DIRECTORY_CHANGE "directory_change"

#define lt_is_directory_change(str) (strcmp(str, LISTENER_TYPE_DIRECTORY_CHANGE) == 0)


typedef struct {
    char **files;
    int count;
    int capacity;
} event_files_t;

typedef struct {
    listener_type_t  type;
    char            *name;
    char            *pattern;
    char            *script;
    bool             cwd;
    media_type_t     media_type;
} event_listener_t;

typedef struct {
    int                version;
    int                listener_count;
    event_listener_t **listeners;
} event_config_t;

extern event_config_t** scan_event_files (const event_files_t *event_files);
extern event_config_t*  scan_event_file (const char *filename);
#endif
