/*
 * memtools - part of WDTV Tools (http://wdtv.elmarweber.org)
 *
 * Copyright (C) 2009 Elmar Weber <wdtv@elmarweber.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * Performs a look up of the current path in the stack memory of the dmaosd
 * process. The adresses are working for dmaosd version 1.02.07 and were
 * determined with the memory_analyzer program.
 * 
 * @author Elmar Weber <wdtv@elmarweber.org>
 */
 
 
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/mman.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ptrace.h>
#include <stdlib.h>
#include <string.h>


/**
 * @param *mem the memory file.
 * @param add the address to read from.
 * 
 * @return the string the char pointer at <code>addr</code> points to.
 */
static char* get_strptr (FILE *mem, unsigned int addr) {
	if (!fseek(mem, addr, SEEK_SET) < 0) 
		perror("fseeking process' memory file");

	unsigned int a;
	fread(&a, sizeof(a), 1, mem);
	if (!fseek(mem, a, SEEK_SET) < 0) 
		perror("fseeking process' memory file");

        char *path = malloc(1024);
	fread(path, 1024, 1, mem);
	return path;
}


int main (int argc, char **argv) {
	FILE *mem = NULL;
	char p[64];
	int pid = 0;
	unsigned long stack_start;
	int location_offset;

	if (argc < 4) {
		fprintf(stderr, "Usage: %s <stack start (16)> <location offset (10)> <pid>\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	
	stack_start = strtoul(argv[1], NULL, 10);
	location_offset = atoi(argv[2]);
	pid = atoi(argv[3]);
	snprintf(p, 63, "/proc/%d/mem", pid);
	if (ptrace(PTRACE_ATTACH, pid, 0, 0))
	    perror("could not attach to process");

	if ( !(mem = fopen(p, "r"))) 
		perror("fopening process' memory file)");

        char *path = get_strptr(mem, (unsigned int) (stack_start + location_offset));
                                
	fclose(mem);
	ptrace(PTRACE_DETACH,pid,0,0);

        printf("%s", path );

        return 0;
}
