/*
 * This file is part of WDTV Tools (http://wdtvtools.sourceforge.net/).
 *
 * Copyright (C) 2009 Elmar Weber <wdtv@elmarweber.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * The implemented functions for the logging system.
 *
 * @author Elmar Weber <wdtv@elmarweber.org>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdbool.h>
#include <stdio.h>
#include <errno.h>


static FILE *_logout = NULL;
static bool use_file = false;

/**
 * Initializes the logging system.
 *
 * @param *log_file the file to log to, NULL if stdout should be used.
 *
 * @return 0 if everything worked fine, an error number like <code>errno</code>
 *         if something went wrong while opening the log file.
 */
extern int log_init(const char *log_file) {
    int result = 0;
    if (log_file == NULL) {
        _logout = stdout;
    } else {
        _logout = fopen(log_file, "w");
        result = errno;
        if (_logout == NULL) {
            _logout = stdout;
        } else {
            use_file = true;
        }
    }
    return result;
}


/**
 * Cleans up all resources from the logging system, closes any open log files.
 */
extern void log_finish () {
    if (use_file) {
        fclose(_logout);
    }
}


/**
 * @return the file pointer for the log output.
 */
extern FILE *logout() {
    return _logout == NULL ? stdout : _logout;
}
