/*
 * This file is part of WDTV Tools (http://wdtvtools.sourceforge.net/).
 *
 * Copyright (C) 2009 Elmar Weber <wdtv@elmarweber.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * Header file that defines macros for the logging system. It must be initialized with
 * log_init() and should ideally be finished with log_finish().
 *
 * Valid log commands are:
 *
 * log_debug
 * log_info
 * log_warn
 * log_error
 * log_fatal
 *
 * It further defines the output stream *logout() that can be used to output further
 * data to the logging system.
 *
 * @author Elmar Weber <shoutcastfs@elmarweber.org>
 */


#ifndef LOG_H_
#define LOG_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <errno.h>


extern int log_init(const char *log_file);
extern void log_finish ();
extern FILE* logout();


#if DEBUG
#define log_debug(format, ...) \
	fprintf(logout(), "DEBUG: " format "\n", ##__VA_ARGS__); \
	fflush(logout());
#else
#define log_debug(format, ...) \
	;
#endif

#define log_info(format, ...) \
	fprintf(logout(), " INFO: " format "\n", ##__VA_ARGS__); \
    fflush(logout());

#define log_warn(format, ...) \
	fprintf(logout(), " WARN: " format "\n", ##__VA_ARGS__); \
    fflush(logout());

#define log_error(format, ...) \
	fprintf(logout(), "ERROR: " format "\n", ##__VA_ARGS__); \
    fflush(logout());

#define log_fatal(format, ...) \
	fprintf(logout(), "FATAL: " format "\n", ##__VA_ARGS__); \
    fflush(logout());

#endif /* LOG_H_ */
