#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>

#include "log.h"


static unsigned int heap_start = 0, heap_end = 0, stack_start = 0, stack_end = 0;


extern unsigned int get_heap_start () {
    return heap_start;
}
    

extern unsigned int get_heap_end () {
    return heap_end;
}
    

extern unsigned int get_stack_start () {
    return stack_start;
}
    

extern unsigned int get_stack_end () {
    return stack_end;
}


static unsigned int get_start_mem (const char* line) {
    char *dash = index(line, '-');
    if (NULL == dash)
        return 0;
    int length = dash - line;
    char *tmp = malloc(length + 1);
    memcpy(tmp, line, length);
    tmp[length] = '\0';
    unsigned long addr = strtoul(tmp, NULL, 16);
    free(tmp);
    return (unsigned int) addr;
}


static unsigned int get_end_mem (const char* line) {
    char *dash = index(line, '-');
    if (NULL == dash)
        return 0;
    int length = dash - line;
    char *tmp = malloc(length + 1);
    memcpy(tmp, dash + 1, length);
    tmp[length] = '\0';
    unsigned long addr = strtoul(tmp, NULL, 16);
    free(tmp);
    return (unsigned int) addr;
}


extern int update_memory_addresses (int pid) {
    char tmp[128], line[1024];
    snprintf(tmp, 128, "/proc/%d/maps", pid);
    FILE *file = fopen(tmp, "r");
    if (NULL == file) {
        log_error("Could not open maps file %s: %s", tmp, strerror(errno));
        return -1;
    }
    while (fgets(line, 1024, file) != NULL) {
        if (strstr(line, "[heap]") != NULL) {
            heap_start = get_start_mem(line);
            heap_end = get_end_mem(line);
        } else if (strstr(line, "[stack]") != NULL) {
            stack_start = get_start_mem(line);
            stack_end = get_end_mem(line);
        }
    }
    if (ferror(file)) {
        log_error("Could not read from maps file: %s", strerror(errno));
    }
    fclose(file);
    
    if (heap_start == 0 || heap_end == 0) {
        log_error("Could not detect heap memory area");
        return -1;
    } else if (stack_start == 0 || stack_end == 0) {
        log_error("Could not detect heap memory area");
        return -1;
    }
    
    log_debug("updated memory:  heap: %#x-%#x, stack: %#x-%#x",
            heap_start, heap_end, stack_start, stack_end);

    return 0;
}
